/*
   (c) 2021 Christian Grieger
   GNU GENERAL PUBLIC LICENSE
*/

#include <Crossing.h>

const byte Crossing::CLOSED  = 0;
const byte Crossing::OPENING = 1;
const byte Crossing::OPEN    = 2;
const byte Crossing::CLOSING = 3;

Crossing::Crossing (byte irLedPin, byte irSensorPin, byte ledPin, byte servoPin, byte openTime)
{
  pinIrLed = irLedPin;
  pinIrSensor = irSensorPin;
  pinLed = ledPin;
  pinServo = servoPin;

  pinMode(pinServo, OUTPUT);

  pinMode(pinIrLed, OUTPUT);
  digitalWrite(pinIrLed, HIGH);

  pinMode(pinLed, OUTPUT);
  digitalWrite(pinLed, LOW);

  pinMode(pinIrSensor, INPUT);

  angle = 90;
  angleSet = 90;
  openDelay = 1000 * openTime;

  setServoAngle(angle);

  crossingState = Crossing::OPEN;
  nextChange = 0;
  nextBlink = 0;
  nextMove = 0;
}

void Crossing::setServoValues(int min, int max)
{
  servoMin = min;
  servoMax = max;
}

void Crossing::setServoAngle(byte angle)
{
  // calculate angle to delayTime
  int delayTime = map(angle, 0, 180, servoMin, servoMax);

  digitalWrite(pinServo, HIGH);
  delayMicroseconds(delayTime);
  digitalWrite(pinServo, LOW);
}

void Crossing::update()
{
  switch (crossingState) {
    case Crossing::OPEN:
      if (analogRead(pinIrSensor) < irSensorOffset) {
        crossingState = Crossing::CLOSING;
        angleSet = 0;
      }
      break;

    case Crossing::CLOSING:
      if ((long)millis() - nextBlink >= 0) {
        nextBlink = (long)millis() + ledBlinkDelay;
        digitalWrite(pinLed, !digitalRead(pinLed));
      }
      if ((long)millis() - nextMove >= 0) {
        nextMove = (long)millis() + servoDelay;
        angle--;
        setServoAngle(angle);
        if (angle <= angleSet) {
          crossingState = Crossing::CLOSED;
          digitalWrite(pinLed, HIGH);
          nextChange = (long)millis() + openDelay;
        }
      }
      break;

    case Crossing::OPENING:
      digitalWrite(pinLed, LOW);
      if ((long)millis() - nextMove >= 0) {
        nextMove = (long)millis() + servoDelay;
        angle++;
        setServoAngle(angle);
        if (angle >= angleSet) {
          crossingState = Crossing::OPEN;
        }
      }
      break;

    case Crossing::CLOSED:
      if ((long)millis() - nextChange >= 0) {
        crossingState = Crossing::OPENING;
        angleSet = 90;
      }
      break;
  }
}
