/*
   (c) 2021 Christian Grieger
   GNU GENERAL PUBLIC LICENSE
*/

#ifndef CROSSING_H
#define CROSSING_H

#include <Arduino.h>

class Crossing
{
  public:
    /*
       Constructor which resets all relevant values and connects the LED to the pin of the Arduino.
    */
    Crossing (byte irLedPin, byte irSensorPin, byte ledPin, byte servoPin, byte openTime);

    /*
       Sets initial values for the connected servo(s)
     */
    void setServoValues(int min, int max);

    /*
       Update routine which should be called in the main loop()-function
    */
    void update();

  private:
    static const byte CLOSED;
    static const byte OPENING;
    static const byte OPEN;
    static const byte CLOSING;

    const int ledBlinkDelay = 250;
    const int servoDelay = 50;
    int openDelay;

    byte irSensorOffset = 10;

    byte pinIrLed;
    byte pinIrSensor;
    byte pinLed;
    byte pinServo;

    long nextChange;
    long nextBlink;
    long nextMove;

    int angle;
    int angleSet;
    int servoMin = 544;
    int servoMax = 2400;

    byte crossingState;

    void setServoAngle(byte angle);
};

#endif
